#include <reg51.h>
#include <stdio.h>

//MakeWiz -> Components Tab -> Add Source File -> ENG_I2C.s51
#include "ENG_I2C.h"    // Low-Level Routines

//Hubungan DT-51 LCMS dengan SPC DC MOTOR
//Hubungkan VMotor pada modul SPC DC MOTOR dengan power supply +12VDC
// jika menggunakan Motor DC 12 volt
#asm
  .export SDA,SCL
  SDA=P3.2            //Pin SDA = P3.2
  SCL=P3.3            //Pin SCL = P3.3
#endasm

//Hubungan DT-51 LCMS dengan PC
//Pin P3.1 (TXD) --- converter --- RX (DB9 pin 2)
//Pin P3.0 (RXD) --- converter --- TX (DB9 pin 3)
//GND --- GND (DB9 pin 5)

near unsigned char GateTime, PWM1, DCControl;
near unsigned int Input1;

#define ADDR_SPC    0xEE        //Address SPC DC MOTOR

void init232(void)      //9600 bps, 8N1
{
    TMOD=(TMOD&0x0F)|0x20;  //Timer 1 Mode 2 (Auto reload)
    SCON=0x52;              //Serial Mode 1 (8-bit UART), REN=1 (Receiver Enable)
                            // TI=1 (Transmit buffer empty)
    PCON=PCON&0x7F;         //SMOD=0 (Double baud rate disable)
    TH1=0xFD;               //9600 bps (11,059 MHz)
    ET1=0;                  //Disable Timer 1 interrupt
    TR1=1;                  //Timer 1 Run
}

//Prosedur untuk inisialisasi SPC DC MOTOR
void DCInit (unsigned char addr, unsigned char GT, unsigned char PWM1,
             unsigned char PWM2, unsigned char DCCtrl)
{
    i2c_sendstart();
    i2c_setbyte(addr&0xFE);     //address SPC DC MOTOR (I2C Write)
    i2c_setbyte(0x00);
    i2c_setbyte(GT);
    i2c_setbyte(PWM1);
    i2c_setbyte(PWM2);
    i2c_setbyte(DCCtrl);
    i2c_sendstop();
}

//Prosedur untuk mengirimkan nilai GateTime
void SetGateTime (unsigned char addr, unsigned char GT)
{
    i2c_sendstart();
    i2c_setbyte(addr&0xFE);
    i2c_setbyte(0x10);          //Command GateTime
    i2c_setbyte(GT);            //GateTime
    i2c_sendstop();
}

//Prosedur untuk mengirimkan nilai PWM1
void SetPWM1 (unsigned char addr, unsigned char PWM1)
{
    i2c_sendstart();
    i2c_setbyte(addr&0xFE);
    i2c_setbyte(0x20);          //Command PWM1
    i2c_setbyte(PWM1);          //PWM1
    i2c_sendstop();
}

//Prosedur untuk mengirimkan nilai PWM2
void SetPWM2 (unsigned char addr, unsigned char PWM2)
{
    i2c_sendstart();
    i2c_setbyte(addr&0xFE);
    i2c_setbyte(0x30);          //Command PWM2
    i2c_setbyte(PWM2);          //PWM2
    i2c_sendstop();
}

//Prosedur untuk mengirimkan nilai DCControl
void SetControl (unsigned char addr, unsigned char DCCtrl)
{
    i2c_sendstart();
    i2c_setbyte(addr&0xFE);
    i2c_setbyte(0x40);          //Command Control
    i2c_setbyte(DCCtrl);        //DC Control
    i2c_sendstop();
}

unsigned int GetInput1 (unsigned char addr)
{
    unsigned int input;
    i2c_sendstart();
    i2c_setbyte(addr&0xFE);
    i2c_setbyte(0x50);          //Command Input
    i2c_setbyte(0x00);          //Input 1
    i2c_sendstop();

    i2c_sendstart();
    i2c_setbyte(addr|0x01);     //address SPC DC MOTOR (I2C Read)
    input=i2c_getbytew();       //Ambil InputL1 + kirim Ack
    input|=i2c_getbytew()<<8;   //Ambil InputH1 + kirim Nack
    i2c_sendstop();
    return input;
}

unsigned int GetInput2 (unsigned char addr)
{
    unsigned int input;
    i2c_sendstart();
    i2c_setbyte(addr&0xFE);
    i2c_setbyte(0x50);          //Command Input
    i2c_setbyte(0x02);          //Input 2
    i2c_sendstop();

    i2c_sendstart();
    i2c_setbyte(addr|0x01);     //address SPC DC MOTOR (I2C Read)
    input=i2c_getbytew();       //Ambil InputL2 + kirim Ack
    input|=i2c_getbytew()<<8;   //Ambil InputH2 + kirim Nack
    i2c_sendstop();
    return input;
}

unsigned int Brake1 (unsigned char addr, unsigned int Brake)
{
    unsigned int Input;
    unsigned char temp;
    SetGateTime(addr,0x01);
    temp=DCControl & 0x02;                  //Ambil DCControl bit ke 1
    if (temp==0x02) temp=DCControl & 0xFD;  //Complement bit ke 1 DCControl
    else temp=DCControl | 0x02;
    SetControl(addr,temp);
    Input=GetInput1(addr);

    while (Input <= Brake)
      {
        for (temp=0; temp<255; temp++);
        Input=GetInput1(addr);
      }
    SetControl(addr,DCControl | 0x01);      //Isi DCControl ke nilai semula (DC1 Stop)
    SetGateTime(addr,GateTime);             //Isi GateTime ke nilai semula
    return Input;
}

unsigned int Brake2 (unsigned char addr, unsigned int Brake)
{
    unsigned int Input;
    unsigned char temp;
    SetGateTime(addr,0x01);
    temp=DCControl & 0x08;                  //Ambil DCControl bit ke 3
    if (temp==0x08) temp=DCControl & 0xF7;  //Complement bit ke 3 DCControl
    else temp=DCControl | 0x08;
    SetControl(addr,temp);
    Input=GetInput1(addr);

    while (Input <= Brake)
      {
        for (temp=0; temp<255; temp++);
        Input=GetInput2(addr);
      }
    SetControl(addr,DCControl | 0x04);      //Isi DCControl ke nilai semula (DC2 Stop)
    SetGateTime(addr,GateTime);             //Isi GateTime ke nilai semula
    return Input;
}

void main (void)
{
    init232();
    _wait_ms(1000);
    PWM1=0;                     //Motor DC 1 berjalan dengan PWM 100%, Ton = 32ms dan Toff = 0ms
    SetPWM1(ADDR_SPC,PWM1);
    GateTime=0x40;              //GateTime= 40H; Time= 1000ms, Resolusi= 1Hz
    SetGateTime(ADDR_SPC,GateTime);
    DCControl=0x44;             //DCControl= 44H; Motor DC 1: Run, CCW, Input aktif
    SetControl(ADDR_SPC,DCControl);
    _wait_ms(1000);

    for (PWM1=0; PWM1<255; PWM1++)
      {
        SetPWM1(ADDR_SPC,PWM1);
        Input1=GetInput1(ADDR_SPC);
        printf("Input 1 = %u\n", Input1);
        _wait_ms(200);
      }
    _wait_ms(1000);

    PWM1=0;
    SetPWM1(ADDR_SPC,PWM1);     //PWM 100%
    DCControl=0x46;             //DCControl= 46H; Motor DC 1: Run, CW, Input aktif
    SetControl(ADDR_SPC,DCControl);
    _wait_ms(2000);
    Brake1(ADDR_SPC,0);

    DCControl=0x05;             //DCControl= 05H; Motor DC 1 dan DC 2: Stop, CCW, Input non-aktif
    SetControl(ADDR_SPC,DCControl);
    while(1);
}